package fr.drools.serialisation.wm;

import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.util.IdentityHashMap;
import java.util.Map;


import org.kie.api.marshalling.ObjectMarshallingStrategy;

public class IdentityPlaceholderResolverStrategy implements ObjectMarshallingStrategy {

    private Map<Integer, Versionable> ids;
    private Map<Object, Integer> objects = new IdentityHashMap();

    public IdentityPlaceholderResolverStrategy(Map<Integer, Versionable> ids) {
        this.ids = ids;
    }

    public Object read(ObjectInputStream os) throws IOException, ClassNotFoundException {
        int id = os.readInt();
        return  ids.get( id );
    }

    public void write(ObjectOutputStream os, Object object) throws IOException {
        Integer id = objects.get( object );
        if ( id == null ) {
            id = ids.size();
            ids.put( id, (Versionable) object);
            objects.put(  object, id );
        }
        os.writeInt( id );
    }

    public boolean accept(Object object) {
        return true;
    }

    public byte[] marshal(Context context,
                          ObjectOutputStream os,
                          Object object) {
        Integer id = objects.get( object );
        if ( id == null ) {
            id = ids.size();
            ids.put( id, (Versionable) object);
            objects.put(  object, id );
        }
        return intToByteArray( id.intValue() );
    }

    public Object unmarshal(Context context,
                            ObjectInputStream is,
                            byte[] object,
                            ClassLoader classloader ) {
        return ids.get( byteArrayToInt( object ) );
    }

    private byte[] intToByteArray(int value) {
        return new byte[] {
                (byte) ((value >>> 24) & 0xFF),
                (byte) ((value >>> 16) & 0xFF),
                (byte) ((value >>> 8) & 0xFF),
                (byte) (value  & 0xFF) };
    }

    private int byteArrayToInt(byte [] b) {
        return (b[0] << 24)
                + ((b[1] & 0xFF) << 16)
                + ((b[2] & 0xFF) << 8)
                + (b[3] & 0xFF);
    }

    public Context createContext() {
        // no need for context
        return null;
    }

}