package fr.drools.serialisation;

import static org.junit.Assert.assertEquals;

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;

import org.drools.core.time.SessionPseudoClock;
import org.junit.After;
import org.junit.Before;
import org.junit.Test;
import org.kie.api.KieBase;
import org.kie.api.KieBaseConfiguration;
import org.kie.api.KieServices;
import org.kie.api.builder.KieBuilder;
import org.kie.api.builder.KieFileSystem;
import org.kie.api.builder.Message;
import org.kie.api.builder.Message.Level;
import org.kie.api.conf.EventProcessingOption;
import org.kie.api.marshalling.KieMarshallers;
import org.kie.api.marshalling.Marshaller;
import org.kie.api.marshalling.ObjectMarshallingStrategy;
import org.kie.api.runtime.KieSession;
import org.kie.api.runtime.KieSessionConfiguration;
import org.kie.api.runtime.conf.ClockTypeOption;
import org.kie.internal.KnowledgeBaseFactory;

import fr.drools.serialisation.events.SupKEvent;


public class DroolsTest {
	
	String drl =  
	  "package fr.drools.serialisation.rules;\n"
	  +"import fr.drools.serialisation.RattachementPere\n"
	  +"import fr.drools.serialisation.RattachementFils\n" 
	  +"import fr.drools.serialisation.events.SupKEvent\n" 
	  +"import fr.drools.serialisation.events.IEvent\n" 
	  +"import fr.drools.serialisation.events.IWorkingMemoryResilient\n" 
	  +"import java.util.ArrayList\n"
     /* +"rule \"RG_RATACHEMENT\"\n"
      +"when\n"
      +"$rattachementsPere : ArrayList(empty==false) from collect( RattachementPere(eventStatus!=\"TO_RETRACT\") )\n"
      +"then\n"
      +"System.out.println(\"RG_RATACHEMENT\");\n"
      +"end\n"  */
      +"rule \"RG_RATACHEMENT_2\"\n"
	  +"when\n"
	  +"$rattachementsPere : ArrayList(empty==false) from collect( RattachementPere(eventStatus==\"TO_RETRACT\") )\n"
	  +"$rattachementsFils : ArrayList() from collect( RattachementFils(eventStatus==\"TO_RETRACT\") )\n"
	  +"then\n"
	  +"System.out.println(\"RG_RATACHEMENT_2\");\n"
	  +"end\n"  
	  +"rule \"RG_RATACHEMENT_3\"\n"
	  +"when\n"
	  +" $events : ArrayList(empty==false) from\n"
	  +" collect( IEvent(!(this instanceof IWorkingMemoryResilient)\n"
	  +" || (this instanceof SupKEvent && ((SupKEvent)this).codeEvt != \"EVT_K_07\" )))\n"
	  +"then\n"
	  +"System.out.println(\"RG_RATACHEMENT_3\");\n"
	  +"end\n";
	
	
	
	 private KieSession ksession;
	 private KieBase kbase;
	 private FileOutputStream fos;
	 private static String FILE_SESSION_PATH = "./session.ser";
	 private SessionPseudoClock clock;

	 
	 
	   @Before
	    public void initialization() {
		    try {
				fos = new FileOutputStream(new File(FILE_SESSION_PATH));
			} catch (FileNotFoundException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
	        KieFileSystem kfs = KieServices.Factory.get().newKieFileSystem();

	        kfs.write("src/main/resources/fr/drools/serialisation/rules/ratachement.drl", drl);
	     

	        KieBuilder kbuilder = KieServices.Factory.get().newKieBuilder(kfs);

	        kbuilder.buildAll();

	        java.util.List<Message> res = kbuilder.getResults().getMessages(Level.ERROR);

	        assertEquals(res.toString(), 0, res.size());

	        KieBaseConfiguration kbconf = KnowledgeBaseFactory
	                .newKnowledgeBaseConfiguration();

	        kbconf.setOption(EventProcessingOption.STREAM);

	        kbase = KieServices.Factory.get()
	                                   .newKieContainer(kbuilder.getKieModule().getReleaseId())
	                                   .newKieBase(kbconf);

	        KieSessionConfiguration ksconfig = KnowledgeBaseFactory
	                .newKnowledgeSessionConfiguration();
	        ksconfig.setOption(ClockTypeOption.get("pseudo"));

	        ksession = kbase.newKieSession(ksconfig, null);

	        clock = ksession.getSessionClock();
	 
	
	    }
	   
	  /* @After
	    public void clean() {
	        ksession.dispose();
	    }
	    
   @Test
	    public void testSerialisationWithCollect() throws InterruptedException {
	        RattachementPere eventp = new RattachementPere();
	        eventp.setEventStatus("TO_RETRACT");
	        RattachementPere eventp1 = new RattachementPere();
	        eventp1.setEventStatus("RETRACT");
	        RattachementPere eventp2 = new RattachementPere();
	        eventp1.setEventStatus("");
	        ksession.insert(eventp);
	        ksession.insert(eventp1);
	        ksession.insert(eventp2);
	        
	        
	        RattachementFils eventf = new RattachementFils();
	        eventp.setEventStatus("TO_RETRACT");
	        RattachementFils eventf1 = new RattachementFils();
	        eventp1.setEventStatus("RETRACT");
	        RattachementFils eventf2 = new RattachementFils();
	        eventp1.setEventStatus("");
	        ksession.insert(eventf);
	        ksession.insert(eventf1);
	        ksession.insert(eventf2);
	        
	        ksession.fireAllRules();
	        ByteArrayOutputStream baos = new ByteArrayOutputStream();

	        KieMarshallers kMarshallers = KieServices.Factory.get().getMarshallers();
	        ObjectMarshallingStrategy oms = kMarshallers.newIdentityMarshallingStrategy();

	        Marshaller marshaller =

	        kMarshallers.newMarshaller( kbase, new ObjectMarshallingStrategy[]{ oms } );

	        try {
				marshaller.marshall( baos, ksession );
				baos.writeTo(fos);
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}

   		}*/
	        @Test
	     	    public void testSerialisationWithCollectRG3() throws InterruptedException {
	     	        SupKEvent kevent = new SupKEvent();
	     	        kevent.setCodeEvt("EVT_K_07");
	     	        ksession.insert(kevent);
	     	        SupKEvent kevent2 = new SupKEvent();
	     	        kevent.setCodeEvt("EVT_K_08");
	     	        ksession.insert(kevent2);
	     	        
	     	        ksession.fireAllRules();
	     	        ByteArrayOutputStream baos = new ByteArrayOutputStream();

	     	        KieMarshallers kMarshallers = KieServices.Factory.get().getMarshallers();
	     	        ObjectMarshallingStrategy oms = kMarshallers.newIdentityMarshallingStrategy();

	     	        Marshaller marshaller =

	     	        kMarshallers.newMarshaller( kbase, new ObjectMarshallingStrategy[]{ oms } );

	     	        try {
	     				marshaller.marshall( baos, ksession );
	     				baos.writeTo(fos);
	     			} catch (IOException e) {
	     				// TODO Auto-generated catch block
	     				e.printStackTrace();
	     			}
	           
	        }
	        
}
	
